package cz.upol.pja.lecture06.client.rest;

import java.util.List;

import javax.json.JsonArray;
import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.core.GenericType;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

/**
 * Ukazkovy klient pro pristup ke STAGu pres tri ruzna rozhrani
 */
public class RESTfulClient {

	public static void main(String[] args) {
		System.out.println(getPredmetInfoXML("KMI", "PJA"));
		System.out.println(getPredmetInfoJson("KMI", "PJA"));
		System.out.println(getPredmetInfo("KMI", "PJA"));
	}

	/**
	 * Vrati informace o predmetu na dane katadre.
	 *
	 * Ke komunikaci se pouziva format XML a JAXB pro prevod dokumentu
	 * na objekt.
	 */
	public static GetPredmetInfoResponse getPredmetInfoXML(String dept, String courseNumber) {
		Client client = ClientBuilder.newClient();
		Response res = client.target("https://stagservices.upol.cz/ws/services/rest/predmety/getPredmetInfo")
				.queryParam("katedra", dept)
				.queryParam("zkratka", courseNumber)
				.request(MediaType.APPLICATION_XML)
				.get();
		
		// vypsal by se obsah odpovedi (obsah lze ale cist pouze jednou)
		// System.out.println(res.readEntity(String.class));
		return res.readEntity(GetPredmetInfoResponse.class);
	}

	/**
	 * Vraci informace o predmetu na dane katedre ve forme
	 * generickeho objektu.
	 * 
	 * Zobrazi i dalsi informace o stavu zpracovani pozadavku.
	 */
	public static List<?> getPredmetInfoJson(String dept, String courseNumber) {
		Client client = ClientBuilder.newClient();
		Response res = client.target("https://stagservices.upol.cz/ws/services/rest/predmety/getPredmetInfo")
				.queryParam("katedra", dept)
				.queryParam("zkratka", courseNumber)
				.queryParam("outputFormat", "JSON") // specificke pro API STAGu
				.request(MediaType.APPLICATION_JSON_TYPE).get();
		
		System.out.println(res.getHeaderString("Content-type"));
		System.out.println("Status: " + res.getStatus());

		return res.readEntity(JsonArray.class);
	}

	/**
	 * Vraci informace o predmetu na dane katedre.
	 *
	 * Je pouzite mapovani JSON do objektu pres knihovnu genson.
	 */
	public static PredmetInfo getPredmetInfo(String dept, String courseNumber) {
		Client client = ClientBuilder.newClient();
		Response res = client.target("https://stagservices.upol.cz/ws/services/rest/predmety/getPredmetInfo")
				.queryParam("katedra", dept)
				.queryParam("zkratka", courseNumber)
				.queryParam("outputFormat", "JSON") // specificke pro API STAGu
				.request(MediaType.APPLICATION_JSON).get();

		GenericType<List<PredmetInfo>> type = new GenericType<List<PredmetInfo>>() { };
		List<PredmetInfo> result = res.readEntity(type);
		return result.get(0);
	}

}
